#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>

#include <cv.h>
#include <cvaux.h>
#include <highgui.h>

#define MAT_TYPE double
#include "cdtw.h"

/*	--------------------------------------------------------------------------------------------
	Convert an image structure into an array given a color channel.

	@param img input image.
	@param channel color channel to convert into the array.
	@param flip flip the image (i.e. horizontal symmetry) if not 0
	@param arr output array address; memory will be allocated if arr is NULL.

	@return Return the array address in case arr is NULL.
*/
MAT_TYPE* convertIplImage2type(IplImage* img,int channel,int flip,MAT_TYPE* arr)
{
	int i,j;
	MAT_TYPE* it;
	if(arr<=NULL) arr = (MAT_TYPE*)malloc(img->width * img->height*sizeof(MAT_TYPE));
	it = arr;

	for(i=0;i<img->height;i++)
	{
		for(j=0;j<img->width;j++)
		{
			*it++ = flip?(MAT_TYPE)(cvGet2D(img,img->height-1-i,j)).val[channel]:
					(MAT_TYPE)(cvGet2D(img,i,j)).val[channel];
		}
	}
	
	return arr;
}

/*	--------------------------------------------------------------------------------------------
	Display sequence allignment.

	@param img input image.
	@param idSeq1 address of the id array for sequence 1.
	@param idSeq2 address of the id array for sequence 2.
	@param nPairs address of the # of pairs (i.e. size of the id array).
*/
void displaySequenceAlignment(IplImage* img, int* idSeq1, int* idSeq2, int nPairs)
{
	int i;
	int x1,x2,y1,y2;

	for(i=0;i<nPairs;i++)
	{
		x1 = idSeq2[i];		y1 = img->height-1-idSeq1[i];
		x2 = idSeq2[i+1];	y2 = img->height-1-idSeq1[i+1];
		if(abs(x2-x1)>2) x2=x1;		if(abs(y2-y1)>2) y2=y1;
		//cvSet2D(costMatrixImage,img->height-1-idSeq1[i],idSeq2[i],cvScalar(10,255,10,255));
		cvLine(img, cvPoint(x1,y1), cvPoint(x2,y2), CV_RGB(255,200,0), 5,CV_AA,0);
	}

	x1 = idSeq2[0];		y1 = img->height-1-idSeq1[0];
	cvCircle(	img,
				cvPoint(x1,y1),
				10, CV_RGB(255,0,0), 7 ,0,0);

	cvNamedWindow("alignment", CV_WINDOW_AUTOSIZE);
	cvShowImage("alignment", img );
	cvWaitKey(0);
}

/*	--------------------------------------------------------------------------------------------
	Main function.
*/
int main( int argc, char** argv )
{
	const char* imgFilename;			/* image filename */
	IplImage* img = NULL;				/* cost matrix image to load */
	MAT_TYPE* costMatrix = NULL;		/* cost matrix array */
	int width=0,height=0;				/* array dimension */
	int radius = 20;					/* defines the neighborhood in the search for the best starting point */

	int* idSeq1=NULL;					/* sequence 1 */
	int* idSeq2=NULL;					/* sequence 2 */
	int  nPairs=0;						/* # of matching pairs */
	MAT_TYPE totalCost=0;				/* total cumulative cost of the sequence alignment */

	/* parse the command line */
	if(argc<2)
	{
		printf("INPUT ERROR: need the name of the image in the command line\n");
		printf("PRESS ENTER TO EXIT\n");
		getchar();
		return -1;
	}
	imgFilename = argv[1];

	/* load the costmatrix image */
	printf("Load image");
	img = cvLoadImage(imgFilename,1);
	if(img<=NULL)
	{
		printf("ERROR: could not load %s\n",imgFilename);
		printf("PRESS ENTER TO EXIT\n");
		getchar();
		return -2;
	}
	printf("		DONE\n");
	

	/* convert the image into an array */
	costMatrix = convertIplImage2type(img,0,1,NULL);
	width = img->width;
	height = img->height;

	/* sequence alignment using CDTW */
	printf("CDTW");
	totalCost = cdtw(costMatrix, height, width,radius,
					 &idSeq1, &idSeq2, &nPairs);
	printf(": %lf",(double)totalCost);
	printf("		DONE\n");

	/* display sequence allignment */
	displaySequenceAlignment(img, idSeq1, idSeq2, nPairs);
	
	/* free memory */
	if(img>NULL)cvReleaseImage(&img);
	if(costMatrix>NULL)free(costMatrix);
	if(idSeq1>NULL)free(idSeq1);
	if(idSeq2>NULL)free(idSeq2);

	return 0;
}

/*	--------------------------------------------------------------------------------------------
	Main function --> loop on CDTW to see if memory increases.
*/
int main2( int argc, char** argv )
{
	const char* imgFilename;			/* image filename */
	IplImage* img = NULL;				/* cost matrix image to load */
	MAT_TYPE* costMatrix = NULL;		/* cost matrix array */
	int width=0,height=0;				/* array dimension */
	int radius = 1;						/* defines the neighborhood in the search for the best starting point */

	int* idSeq1=NULL;					/* sequence 1 */
	int* idSeq2=NULL;					/* sequence 2 */
	int  nPairs=0;						/* # of matching pairs */
	MAT_TYPE totalCost=0;				/* total cumulative cost of the sequence alignment */

	int iter;
	int nIter = 10000000;				/* loop over CDTW to check the memory */

	/* parse the command line */
	if(argc<2)
	{
		printf("INPUT ERROR: need the name of the image in the command line\n");
		printf("PRESS ENTER TO EXIT\n");
		getchar();
		return -1;
	}
	imgFilename = argv[1];

	/* load the costmatrix image */
	printf("Load image");
	img = cvLoadImage(imgFilename,1);
	if(img<=NULL)
	{
		printf("ERROR: could not load %s\n",imgFilename);
		printf("PRESS ENTER TO EXIT\n");
		getchar();
		return -2;
	}
	printf("		DONE\n");
	

	/* convert the image into an array */
	costMatrix = convertIplImage2type(img,0,1,NULL);
	width = img->width;
	height = img->height;

	/* sequence alignment using CDTW */
	printf("CDTW");

	printf("\nmemory check\n");
	for(iter=1;iter<=nIter;iter++)
	{
		printf("#iter: %d/%d\n",iter,nIter);
		if(idSeq1>NULL)free(idSeq1);
		if(idSeq2>NULL)free(idSeq2);
		totalCost = cdtw(costMatrix, height, width,radius,
						 &idSeq1, &idSeq2, &nPairs);
	}
	printf(": %lf",(double)totalCost);
	printf("		DONE\n");

	/* display sequence allignment */
	displaySequenceAlignment(img, idSeq1, idSeq2, nPairs);
	
	/* free memory */
	if(img>NULL)cvReleaseImage(&img);
	if(costMatrix>NULL)free(costMatrix);
	if(idSeq1>NULL)free(idSeq1);
	if(idSeq2>NULL)free(idSeq2);

	return 0;
}